"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const access_1 = require("../tools/access");
const fs_1 = __importDefault(require("fs"));
class SelectTableCreator {
    constructor(table_name) {
        this.imports = `
import { ConditionType, ConditionGroupType, NumberCondition, StringCondition, DateCondition, GenericSelectQuery, OrderByDirections, Condition } from './databasetool.interfaces'
import { SelectWizard } from './Table.class'
import { TableHelper } from './table.helper.class'
    `;
        this.table_name = table_name;
    }
    get fieldsEnum() {
        return this.createFieldsEnum();
    }
    get tableInterface() {
        return this.createTableInterface();
    }
    get staticRequirements() {
        return `

interface ${this.table_name}SelectQuery {
    fields: Array<${this.table_name}Fields>,
    where: ${this.table_name}ConditionGroup | ${this.table_name}Condition
}

interface ${this.table_name}ConditionGroup {
    type: ConditionGroupType,
    conditions: Array<${this.table_name}ConditionGroup | ${this.table_name}Condition> | ${this.table_name}Condition
}


interface ${this.table_name}SelectQuery {
    fields: Array<${this.table_name}Fields>,
    where: ${this.table_name}ConditionGroup | ${this.table_name}Condition,
    limit?: number,
    orderBy?: ${this.table_name}SelectOrderBy
}

interface ${this.table_name}SelectOrderBy {
    field: ${this.table_name}Fields,
    direction: OrderByDirections
}


let customFunctions: any = {
    someCustom: (input: any): Condition => {
        return {
            field: 'id',
            type: ConditionType.gt,
            value: input + ' AND THRN SOME'
        }
    }
}

export function ${this.table_name}Select(${this.table_name}Select: ${this.table_name}SelectQuery): SelectWizard {
    let genericSelect = ${this.table_name}Select as any as GenericSelectQuery
    let tableHelper = new TableHelper({ base: '${this.table_name}', customGroup: genericSelect, CustomProperties: customFunctions })
    return tableHelper.returnTable()
}

        `;
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            this.table_properties = yield access_1.query(`describe ${this.table_name}`);
        });
    }
    createFieldsEnum() {
        let create = (field) => `${field} = '${field}',`;
        let Fields = [];
        this.table_properties.forEach((column) => {
            Fields.push(create(column.Field));
        });
        let insideEnums = Fields.join(`
        `);
        return `
    export enum ${this.table_name}Fields {
        ${insideEnums}
    }
        `;
    }
    createTableInterface() {
        let create = (field, type) => {
            if (type.includes('int')) {
                return createNumberField(field);
            }
            else if (type.includes('text') || type.includes('varchar')) {
                return createStringField(field);
            }
            else if (type.includes('date') || type.includes('time')) {
                return createDateField(field);
            }
            else {
                console.log('\n\n\nERROR EVERY TYPE SHOULD BE ACCOUNTED FOR\n\n\n');
                return 'ERROR EVERY TYPE SHOULD BE ACCOUNTED FOR';
            }
        };
        let createNumberField = (field) => `${field}?: number | NumberCondition,`;
        let createDateField = (field) => `${field}?: Date | DateCondition,`;
        let createStringField = (field) => `${field}?: String | StringCondition,`;
        let Fields = [];
        this.table_properties.forEach((column) => {
            Fields.push(create(column.Field, column.Type));
        });
        let insideEnums = Fields.join(`
        `);
        return `
    interface ${this.table_name}Condition {
        ${insideEnums}
    }
        `;
    }
    get fullPage() {
        return `
    ${this.imports}
    ${this.fieldsEnum}
    ${this.tableInterface}
    ${this.staticRequirements}
        `;
    }
    writeToFile(filePath) {
        fs_1.default.writeFileSync(filePath, this.fullPage);
    }
}
exports.SelectTableCreator = SelectTableCreator;
//# sourceMappingURL=create.table.select.from.database.js.map