"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const messages_select_class_1 = require("./messages.select.class");
const databasetool_interfaces_1 = require("../tools/databasetool.interfaces");
function normalizeWS(s) {
    s = s.trim();
    s = s.match(/\S+/g);
    return s ? s.join(' ') : '';
}
describe("Should run", () => {
    it("Should be awesome", () => {
        expect(1).toBe(1);
    });
});
describe("Should create new select Messages queries", () => {
    let { id, user, account, date_created, date_sent, body } = messages_select_class_1.MessageFields;
    let returnFromMessageSelect = messages_select_class_1.MessageSelect({
        fields: [id, user, account, date_created, body], where: {
            theirNumber: '+19794020005',
        },
        limit: 7,
        orderBy: {
            field: id,
            direction: databasetool_interfaces_1.OrderByDirections.asc
        }
    });
    it("Should be awesome", () => {
        expect(normalizeWS(returnFromMessageSelect.stringify())).toBe(normalizeWS("Select id , user , account , date_created , body From Messages Where ( theirNumber = '+19794020005' ) Order By id asc Limit 7"));
    });
    //Succeeds but no need to query the database constantly
    xit("should actually query database and return 7 results", () => __awaiter(this, void 0, void 0, function* () {
        let queryResults = yield returnFromMessageSelect.execute();
        expect(queryResults).toHaveLength(7);
    }));
});
describe("Should apply group by conditition", () => {
    let { id, user, account, date_created, date_sent, body } = messages_select_class_1.MessageFields;
    let returnFromMessageSelect = messages_select_class_1.MessageSelect({
        fields: [id, user, account, date_created, body], where: {
            body: {
                type: databasetool_interfaces_1.StringConditionType.like,
                value: 'Hello'
            },
        },
        limit: 100,
        orderBy: {
            field: id,
            direction: databasetool_interfaces_1.OrderByDirections.asc
        },
        groupBy: 'theirnumber'
    });
    it("Should be awesome", () => {
        expect(normalizeWS(returnFromMessageSelect.stringify())).toBe(normalizeWS("Select id , user , account , date_created , body From Messages Where ( body like '%Hello%' ) group by theirnumber Order By id asc Limit 100"));
    });
    //Succeeds but no need to query the database constantly
    xit("should actually query database and return 21 results", () => __awaiter(this, void 0, void 0, function* () {
        let queryResults = yield returnFromMessageSelect.execute();
        expect(queryResults).toHaveLength(21);
    }));
});
describe("Custom properties / functions should work", () => {
    let { id, user, account, date_created, date_sent, body } = messages_select_class_1.MessageFields;
    let returnFromMessageSelect = messages_select_class_1.MessageSelect({
        fields: [id, user, account, date_created, body], where: {
            theirNumber: '+19794020005',
            someCustom: 7
        },
        limit: 7,
        orderBy: {
            field: id,
            direction: databasetool_interfaces_1.OrderByDirections.asc
        }
    });
    it("Should be awesome", () => {
        expect(normalizeWS(returnFromMessageSelect.stringify())).toBe(normalizeWS("Select id , user , account , date_created , body From Messages Where ( theirNumber = '+19794020005' AND id > '7 AND THRN SOME' ) Order By id asc Limit 7"));
    });
});
//# sourceMappingURL=messages.select.test.js.map