"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const select_class_1 = require("../tools/select.class");
const databasetool_interfaces_1 = require("../tools/databasetool.interfaces");
function normalizeWS(s) {
    s = s.match(/\S+/g);
    return s ? s.join(' ') : '';
}
let testCondition1 = {
    type: databasetool_interfaces_1.ConditionType.gt,
    field: "Field 1",
    value: "Some Value"
};
let testCondition2 = {
    type: databasetool_interfaces_1.ConditionType.equals,
    field: "Field 1",
    value: "Some Value"
};
let testCondition3 = {
    type: databasetool_interfaces_1.ConditionType.lt,
    field: "Field 1",
    value: "Some Value"
};
let testCondition4 = {
    type: databasetool_interfaces_1.ConditionType.like,
    field: "Field 1",
    value: "Some Value"
};
let testCondition5 = {
    type: databasetool_interfaces_1.ConditionType.equals,
    field: "dateField",
    value: new Date('06-29-1984')
};
let testCondition6 = {
    type: databasetool_interfaces_1.ConditionType.equals,
    field: "numberField",
    value: 32,
};
let simpleTable = new select_class_1.SelectWizard({
    base: 'Messages',
    fields: [
        'ourNumber',
        'theirnumber',
        'body'
    ],
    where: {
        field: 'theirnumber',
        type: databasetool_interfaces_1.ConditionType.equals,
        value: '+19794020005'
    },
    limit: 2
});
describe('ConditionClass and ConditionGroupClass', () => {
    it("ConditionClass.stringify() return the right string", done => {
        expect(new select_class_1.ConditionClass(testCondition1).stringify()).toBe(" Field 1  >  'Some Value'");
        expect(new select_class_1.ConditionClass(testCondition2).stringify()).toBe(" Field 1  =  'Some Value'");
        expect(new select_class_1.ConditionClass(testCondition3).stringify()).toBe(" Field 1  <  'Some Value'");
        expect(new select_class_1.ConditionClass(testCondition4).stringify()).toBe(` Field 1 like '%Some Value%'`);
        expect(new select_class_1.ConditionClass(testCondition5).stringify()).toBe(` dateField  =  '1984-06-29 00:00:00.000'`);
        expect(new select_class_1.ConditionClass(testCondition6).stringify()).toBe(` numberField  =  32`);
        done();
    });
    it("ConditionGroupClass.stringify() return the right string", done => {
        expect(new select_class_1.ConditionGroupClass({
            type: databasetool_interfaces_1.ConditionGroupType.and,
            conditions: [
                testCondition1,
                testCondition2,
                testCondition4
            ]
        }).stringify()).toBe(`
(
     Field 1  >  'Some Value'
        AND
         Field 1  =  'Some Value'
        AND
         Field 1 like '%Some Value%'
)`);
        expect(normalizeWS(new select_class_1.ConditionGroupClass({
            type: databasetool_interfaces_1.ConditionGroupType.and,
            conditions: [
                testCondition1,
                testCondition2,
                {
                    type: databasetool_interfaces_1.ConditionGroupType.or,
                    conditions: [
                        testCondition1,
                        testCondition2,
                        testCondition4
                    ]
                }
            ]
        }).stringify())).toBe(normalizeWS(`
(
     Field 1  >  'Some Value'
        AND
         Field 1  =  'Some Value'
        AND

(
     Field 1  >  'Some Value'
        OR
         Field 1  =  'Some Value'
        OR
         Field 1 like '%Some Value%'
)
)`));
        done();
    });
});
describe('table class', () => {
    it("Should create the right query", () => {
        let newTable1 = new select_class_1.SelectWizard({
            base: 'Messages',
            fields: [
                'id',
                'account'
            ],
            where: {
                type: databasetool_interfaces_1.ConditionGroupType.and,
                conditions: [
                    testCondition1,
                    testCondition2,
                    {
                        type: databasetool_interfaces_1.ConditionGroupType.or,
                        conditions: [
                            testCondition1,
                            testCondition2,
                            testCondition4
                        ]
                    }
                ]
            },
            groupBy: 'theirnumber'
        });
        expect(normalizeWS(newTable1.stringify())).toBe(normalizeWS(`
            Select
                id , account
            From
                Messages
            Where

    (
         Field 1  >  'Some Value'
            AND
             Field 1  =  'Some Value'
            AND

    (
         Field 1  >  'Some Value'
            OR
             Field 1  =  'Some Value'
            OR
             Field 1 like '%Some Value%'
    )
    )  group by theirnumber Limit 1000
            `));
    });
    xit('Should Query Database', () => __awaiter(this, void 0, void 0, function* () {
        expect(normalizeWS(simpleTable.stringify())).toBe(normalizeWS(`
            Select
                ourNumber , theirnumber , body
            From
                Messages
            Where

    (
         theirnumber  =  '+19794020005'
    )  Limit 2
            `));
        let queryResult = yield simpleTable.execute();
        expect(queryResult.length).toBe(2);
    }));
});
//# sourceMappingURL=select.class.test.js.map