"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class TableCreator {
    constructor(tableName, tableProperties) {
        this.tableName = tableName;
        this.tableProperties = tableProperties;
        this.tableProperties.forEach((el) => {
            if (this.isPrimaryKey(el)) {
                this.primaryKey = el["Field"];
                this.primaryKeyType = this.typeOfRow(el);
            }
        });
    }
    isPrimaryKey(el) {
        return el["Key"] === "PRI";
    }
    typeOfRow(el) {
        return this.typescriptTypeFromMySqlType(el["Type"]);
    }
    typescriptTypeFromMySqlType(mySqlType) {
        if (mySqlType.includes('int') || mySqlType.includes('decimal')) {
            return 'number';
        }
        else if (mySqlType.includes('text') || mySqlType.includes('char')) {
            return 'string';
        }
        else if (mySqlType.includes('date') || mySqlType.includes('time')) {
            return 'Date';
        }
        else {
            console.log('\n\n\nERROR EVERY TYPE SHOULD BE ACCOUNTED FOR\n\n\n', 'TYPE:' + mySqlType + '\n\n');
            return 'ERROR EVERY TYPE SHOULD BE ACCOUNTED FOR';
        }
    }
    get imports() {
        let r = `import { ConditionType, ConditionGroupType, NumberCondition, StringCondition, DateCondition, GenericSelectQuery, OrderByDirections, Condition, GenericOrderBy } from '../tools/databasetool.interfaces'
import { ConvertSpecificTableSelectIntoGeneralSelectWizard } from '../tools/select.helper.class'
import { InstanceWizard } from '../tools/instance.class'

`;
        return r;
    }
    get fieldsEnum() {
        let create = (field) => `${field} = '${field}',`;
        let Fields = [`star = '*',`];
        this.tableProperties.forEach((column) => {
            Fields.push(create(column.Field));
        });
        let insideEnums = Fields.join(`
        `);
        return `
    export enum ${this.tableName}Fields {
        ${insideEnums}
    }
        `;
    }
    get primaryKeyString() {
        if (this.primaryKey) {
            return `let primary_key: ${this.tableName}Fields = ${this.tableName}Fields['${this.primaryKey}']
`;
        }
        else {
            return '';
        }
    }
    get condition() {
        let create = (field, type) => {
            if (type === 'number') {
                return createNumberField(field);
            }
            else if (type === 'string') {
                return createStringField(field);
            }
            else if (type === 'Date') {
                return createDateField(field);
            }
            else {
                console.log('\n\n\nERROR EVERY TYPE SHOULD BE ACCOUNTED FOR\n\n\n');
                return 'ERROR EVERY TYPE SHOULD BE ACCOUNTED FOR';
            }
        };
        let createNumberField = (field) => `${field}?: number | NumberCondition,`;
        let createDateField = (field) => `${field}?: Date | DateCondition,`;
        let createStringField = (field) => `${field}?: string | StringCondition,`;
        let Fields = [];
        this.tableProperties.forEach((column) => {
            let type = this.typescriptTypeFromMySqlType(column.Type);
            Fields.push(create(column.Field, type));
        });
        let insideEnums = Fields.join(`
        `);
        let r = `
interface ${this.tableName}Condition {
        ${insideEnums}
}
`;
        return r;
    }
    get simpleInterfacesAndCustomFunctions() {
        let r = `  
interface ${this.tableName}ConditionGroup {
    type: ConditionGroupType,
    conditions: Array<${this.tableName}ConditionGroup | ${this.tableName}Condition> | ${this.tableName}Condition
}


interface ${this.tableName}SelectOrderBy {
    field: ${this.tableName}Fields,
    direction: OrderByDirections
}

interface ${this.tableName}SelectGroupBy {
    field: ${this.tableName}Fields
}

let customFunctions: any = {
    someCustom: (input: any): Condition => {
        return {
            field: 'id',
            type: ConditionType.gt,
            value: input + ' AND THRN SOME'
        }
    }
}
`;
        return r;
    }
    get interface() {
        let create = (field, type) => {
            if (type === 'number') {
                return createNumberField(field);
            }
            else if (type === 'string') {
                return createStringField(field);
            }
            else if (type === 'Date') {
                return createDateField(field);
            }
            else {
                console.log('\n\n\nERROR EVERY TYPE SHOULD BE ACCOUNTED FOR\n\n\n');
                return 'ERROR EVERY TYPE SHOULD BE ACCOUNTED FOR';
            }
        };
        let createNumberField = (field) => `${field}?: number,`;
        let createDateField = (field) => `${field}?: Date,`;
        let createStringField = (field) => `${field}?: String,`;
        let Fields = [];
        this.tableProperties.forEach((column) => {
            let type = this.typescriptTypeFromMySqlType(column.Type);
            Fields.push(create(column.Field, type));
        });
        let insideEnums = Fields.join(`
        `);
        let r = `
interface ${this.tableName}Interface {
        ${insideEnums}
}
        `;
        return r;
    }
    get tableClass() {
        let r = `
export default class ${this.tableName}Class {

fields = ${this.tableName}Fields

new(${this.tableName}?: ${this.tableName}Interface): ${this.tableName}Instance {
    return new ${this.tableName}Instance(${this.tableName})
}

async select(${this.tableName}Select: {
    fields: Array<${this.tableName}Fields>,
    where: ${this.tableName}ConditionGroup | ${this.tableName}Condition,
    limit?: number,
    orderBy?: ${this.tableName}SelectOrderBy,
    groupBy?: ${this.tableName}SelectGroupBy
}): Promise<Array<any>> {
    let genericSelect = ${this.tableName}Select as any as GenericSelectQuery
    if (!genericSelect.fields.includes(primary_key)) ${this.tableName}Select.fields.push(primary_key)
    let tableHelper = new ConvertSpecificTableSelectIntoGeneralSelectWizard({ base: '${this.tableName}', customGroup: genericSelect, CustomProperties: customFunctions })
    let anyGenericResults = await tableHelper.returnTable().execute()
    let arrayOfGenericResults = anyGenericResults as any as Array<any>
    let arrayOfInstances: Array<${this.tableName}Instance> = arrayOfGenericResults.map(l => new ${this.tableName}Instance(l))
    return arrayOfInstances
}

async upsert(${this.tableName}: ${this.tableName}Interface | ${this.tableName}Instance | Array<${this.tableName}Interface | ${this.tableName}Instance>): Promise<any> {
    let save = (single: any): Promise<any> => {
        if (single.is${this.tableName}Instance)
            return single.save()
        else
            return new ${this.tableName}Instance(single).save()
    }
    if (Array.isArray(${this.tableName})) {
        return Promise.all(${this.tableName}.map(el => save(el)))
    } else {
        return save(${this.tableName})
    }

}


async delete(${this.tableName}: ${this.tableName}Interface | ${this.tableName}Instance | Array<${this.tableName}Interface | ${this.tableName}Instance>): Promise<any> {
    let remove = (single: any): Promise<any> => {
        if (single.is${this.tableName}Instance)
            return single.delete()
        else
            return new ${this.tableName}Instance(single).delete()
    }


    if (Array.isArray(${this.tableName})) {
        return Promise.all(${this.tableName}.map(el => remove(el)))
    } else {
        return remove(${this.tableName})
    }
}

${this.findByIdString}

}
`;
        return r;
    }
    get findByIdString() {
        if (this.primaryKey && this.primaryKeyType === 'number') {
            return `
        //Only shown if the primary key is a number
async findById(id: number) {
    let newCondition: any = {}
    newCondition[primary_key] = id
    newCondition as ${this.tableName}Condition

    let resultOffind = await this.select({
        fields: [${this.tableName}Fields.star],
        where: newCondition
    })

    if (resultOffind.length && resultOffind.length === 1) return resultOffind[0]
    else return {
        error: "Should have returned an array with exactly 1 result, however it did not. please see the results property",
        results: resultOffind
    }
}
            `;
        }
        else {
            return '';
        }
    }
    get instanceClass() {
        return `
class ${this.tableName}Instance {
    readonly is${this.tableName}Instance: boolean = true
    constructor(${this.tableName}?: ${this.tableName}Interface) {
        if (!${this.tableName}) ${this.tableName} = {}
        let genericObj = ${this.tableName} as any
        this.instance = new InstanceWizard({ table: '${this.tableName}', primary_key: 'id', fields: genericObj })
    }
    private instance: InstanceWizard

    ${this.createGettersAndSettersForInstance}

    async save(): Promise<any> {
        return await this.instance.save()
    }

    async delete(): Promise<any> {
        return await this.instance.delete()
    }
}
        
        `;
    }
    get createGettersAndSettersForInstance() {
        let gettersAndSetters = [];
        this.tableProperties.forEach((el) => {
            let field = el["Field"];
            let type = this.typeOfRow(el);
            if (type === 'Date')
                type = 'Date | string';
            if (this.isPrimaryKey(el)) {
                gettersAndSetters.push(`
get ${field}() { return this.instance.fields['${field}'] }
set ${field}(val: ${type}) {
    if (this.instance.isNew) {
        this.instance.fields['${field}'] = val
    } else {
        console.log('YOU CAN NOT CHANGE THE PRIMARY KEY OF AN EXISTING ROW')
    }
}
                `);
            }
            else {
                gettersAndSetters.push(`
get ${field}(): ${type} {
    return this.instance.fields['${field}']
}

    set ${field}(val: ${type}) {
    this.instance.fields['${field}'] = val
}
                `);
            }
        });
        return gettersAndSetters.join(`
        `);
    }
    get fullPage() {
        return `
    ${this.imports}
    ${this.fieldsEnum}
    ${this.primaryKeyString}
    ${this.condition}
    ${this.simpleInterfacesAndCustomFunctions}
    ${this.interface}
    ${this.tableClass}
    ${this.instanceClass}
        `;
    }
}
exports.TableCreator = TableCreator;
//# sourceMappingURL=table.creator.class.js.map