"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const master_creator_class_1 = __importDefault(require("./creator/master.creator.class"));
const table_creator_class_1 = require("./creator/table.creator.class");
const fs_1 = __importDefault(require("fs"));
class mysqlTypescript {
    constructor(db, _directory, databaseSchemaName) {
        this.db = db;
        this._directory = _directory;
        this.databaseSchemaName = databaseSchemaName;
        this.createFoldersIfnotExists();
        this.copyToolsFolder();
    }
    getTableNames() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._tableNames)
                return this._tableNames;
            let tableNames = yield this.query(`
        SELECT table_name 
        FROM information_schema.tables 
        WHERE table_schema = '${this.databaseSchemaName}'
        `);
            this._tableNames = tableNames.map((tableName) => tableName.table_name);
            return this._tableNames;
        });
    }
    createFoldersIfnotExists() {
        this.checkAndMakeDir(`${this._directory}/database`);
        this.checkAndMakeDir(`${this._directory}/database/tables`);
        this.checkAndMakeDir(`${this._directory}/database/tools`);
        this.checkAndMakeDir(`${this._directory}/database/functions`);
    }
    copyToolsFolder() {
        let arrayOfTools = fs_1.default.readdirSync(`${__dirname}/tools`).map(file => ({
            source: `${__dirname}/tools/${file}`,
            destination: `${this._directory}/database/tools/${file}`
        }));
        for (let tool of arrayOfTools) {
            fs_1.default.copyFileSync(tool.source, tool.destination);
        }
    }
    query(q, parameters) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                this.db.query(q, parameters, (error, response) => {
                    if (!error) {
                        resolve(response);
                    }
                    else {
                        reject(error);
                    }
                });
            });
        });
    }
    checkAndMakeDir(dir) {
        return __awaiter(this, void 0, void 0, function* () {
            if (fs_1.default.existsSync(dir))
                return;
            else
                fs_1.default.mkdirSync(dir);
        });
    }
    CreateTable(tableName) {
        return __awaiter(this, void 0, void 0, function* () {
            let tableProperties = yield this.query(`Describe ${tableName}`);
            let fullPageString = new table_creator_class_1.TableCreator(tableName, tableProperties).fullPage;
            let pathToWrite = `${this._directory}/database/tables/${tableName}.class.ts`;
            return fs_1.default.writeFileSync(pathToWrite, fullPageString);
        });
    }
    CreateAllTables() {
        return __awaiter(this, void 0, void 0, function* () {
            let tableNames = yield this.getTableNames();
            for (let table of tableNames) {
                yield this.CreateTable(table);
            }
        });
    }
    CreateFunctions() {
        return __awaiter(this, void 0, void 0, function* () {
            return;
        });
    }
    CreateMaster() {
        return __awaiter(this, void 0, void 0, function* () {
            let tables = yield this.getTableNames();
            let nmc = new master_creator_class_1.default(tables);
            let pathToWrite = `${this._directory}/database/master.class.ts`;
            fs_1.default.writeFileSync(pathToWrite, nmc.fullPage);
        });
    }
    CreateAll() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.CreateAllTables();
            yield this.CreateFunctions();
            return yield this.CreateMaster();
        });
    }
}
exports.mysqlTypescript = mysqlTypescript;
//# sourceMappingURL=index.js.map