"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const databasetool_interfaces_1 = require("./databasetool.interfaces");
const access_1 = require("./access");
class SelectWizard {
    constructor(selectWizardProperties) {
        this.limit = 1000;
        if (ConditionGroupClass.isConditionGroup(selectWizardProperties.where))
            this.conditions = new ConditionGroupClass(selectWizardProperties.where);
        else
            this.conditions = new ConditionGroupClass({
                type: databasetool_interfaces_1.ConditionGroupType.and,
                conditions: [selectWizardProperties.where]
            });
        this.fields = selectWizardProperties.fields;
        this.base = selectWizardProperties.base;
        this.orderBy = selectWizardProperties.orderBy;
        this.groupBy = selectWizardProperties.groupBy;
        if (selectWizardProperties.limit)
            this.limit = selectWizardProperties.limit;
    }
    fieldsString() { return this.fields.join(' , '); }
    getOrderByString() {
        if (this.orderBy) {
            let { field, direction } = this.orderBy;
            return ` Order By ${field} ${direction} `;
        }
        else
            return '';
    }
    getGroupByString() {
        if (this.groupBy) {
            return ` group by ${this.groupBy} `;
        }
        else {
            return '';
        }
    }
    execute() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield access_1.query(this.stringify());
        });
    }
    stringify() {
        return `
        Select
            ${this.fieldsString()}
        From
            ${this.base}
        Where
            ${this.conditions.stringify()}
        ${this.getGroupByString()}
        ${this.getOrderByString()}
        Limit
            ${this.limit}
        `;
    }
}
exports.SelectWizard = SelectWizard;
class ConditionGroupClass {
    constructor(conditionGroup) {
        this.conditions = [];
        this.type = conditionGroup.type;
        conditionGroup.conditions.forEach(condition => {
            if (ConditionGroupClass.isConditionGroup(condition))
                this.conditions.push(new ConditionGroupClass(condition));
            else
                this.conditions.push(new ConditionClass(condition));
        });
    }
    static isConditionGroup(arg) {
        return arg.conditions ? true : false;
    }
    returnArrayOfConditionStrings() {
        return this.conditions.map(condition => condition.stringify());
    }
    stringify() {
        let arrayOfConditionStrings = this.returnArrayOfConditionStrings();
        let joinedConditions = arrayOfConditionStrings.join(`
        ${this.type}
        `);
        return `
(
    ${joinedConditions}
)`;
    }
}
exports.ConditionGroupClass = ConditionGroupClass;
class ConditionClass {
    constructor(condition) {
        this.condition = condition;
    }
    stringify() {
        switch (this.condition.type) {
            case databasetool_interfaces_1.ConditionType.equals:
            case databasetool_interfaces_1.ConditionType.gt:
            case databasetool_interfaces_1.ConditionType.lt:
            case databasetool_interfaces_1.ConditionType.ltoe:
            case databasetool_interfaces_1.ConditionType.gtoe:
                return ` ${this.condition.field} ${this.condition.type} ${access_1.db.escape(this.condition.value)}`;
                break;
            case databasetool_interfaces_1.ConditionType.like:
                return ` ${this.condition.field} like ${access_1.db.escape(`%${this.condition.value}%`)}`;
                break;
            default:
                return 'SOMETHING WENT WRONG';
                break;
        }
    }
}
exports.ConditionClass = ConditionClass;
//# sourceMappingURL=select.class.js.map