"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const databasetool_interfaces_1 = require("./databasetool.interfaces");
const select_class_1 = require("./select.class");
class ConvertSpecificTableSelectIntoGeneralSelectWizard {
    constructor(CreatorParameters) {
        this.base = CreatorParameters.base;
        this.fields = CreatorParameters.customGroup.fields;
        this.customFunctions = CreatorParameters.CustomProperties || {};
        this.where = this.generateWhere(CreatorParameters.customGroup.where);
        this.limit = CreatorParameters.customGroup.limit;
        this.orderBy = CreatorParameters.customGroup.orderBy;
        this.groupBy = CreatorParameters.customGroup.groupBy;
    }
    get selectWizardProperties() {
        return {
            base: this.base,
            fields: this.fields,
            where: this.where,
            limit: this.limit,
            orderBy: this.orderBy,
            groupBy: this.groupBy
        };
    }
    generateWhere(inputWhere) {
        return this.parseConditionOrConditionGroup(inputWhere);
    }
    parseConditionOrConditionGroup(conditionOrConditionGroup, parentConditionGroup) {
        let isConditionGroup = select_class_1.ConditionGroupClass.isConditionGroup(conditionOrConditionGroup);
        if (isConditionGroup && !parentConditionGroup) {
            return this.parseCondtionGroup(conditionOrConditionGroup);
        }
        else if (isConditionGroup && parentConditionGroup) {
            parentConditionGroup.conditions.push(this.parseCondtionGroup(conditionOrConditionGroup));
            return parentConditionGroup;
        }
        else if (!isConditionGroup && !parentConditionGroup) {
            return this.returnNewCondtionGroup(conditionOrConditionGroup);
        }
        else if (!isConditionGroup && parentConditionGroup) {
            let ReturnCOnditos = this.parseCondtionToArrayOfConditions(conditionOrConditionGroup);
            parentConditionGroup.conditions = parentConditionGroup.conditions.concat(ReturnCOnditos);
            return parentConditionGroup;
        }
        else {
            console.log('Should never be here, conditions should be met.');
        }
    }
    returnNewCondtionGroup(condition) {
        return {
            type: databasetool_interfaces_1.ConditionGroupType.and,
            conditions: this.parseCondtionToArrayOfConditions(condition)
        };
    }
    parseCondtionGroup(conditionGroup) {
        let newConditionGroup = {
            type: conditionGroup.type,
            conditions: []
        };
        conditionGroup.conditions.forEach((conditionOrConditionGroup) => {
            this.parseConditionOrConditionGroup(conditionOrConditionGroup, newConditionGroup);
        });
        return newConditionGroup;
    }
    parseCondtionToArrayOfConditions(conditionGroup) {
        let isGenericCondition = (n) => n.type;
        let newCondtionArray = [];
        for (let prop in conditionGroup) {
            if (this.customFunctions[prop])
                newCondtionArray.push(this.customFunctions[prop](conditionGroup[prop]));
            else if (isGenericCondition(conditionGroup[prop])) {
                newCondtionArray.push({
                    field: prop,
                    type: conditionGroup[prop].type,
                    value: conditionGroup[prop].value
                });
            }
            else {
                newCondtionArray.push({
                    field: prop,
                    type: databasetool_interfaces_1.ConditionType.equals,
                    value: conditionGroup[prop]
                });
            }
        }
        return newCondtionArray;
    }
    returnTable() {
        return new select_class_1.SelectWizard(this.selectWizardProperties);
    }
}
exports.ConvertSpecificTableSelectIntoGeneralSelectWizard = ConvertSpecificTableSelectIntoGeneralSelectWizard;
//     static CustomCreator(CreatorParameters: { base: string, customGroup: GenericSelectQuery, CustomProperties?: any, limit?: number }): SelectWizardProperties {
//     let { CustomProperties } = CreatorParameters
//     if (!CustomProperties) CustomProperties = {}
//     function returnConditionsArrayFromObject(obj: any): Array<Condition> {
//         let isGenericCondition = (n: any): boolean => n.type
//         let newCondtionArray: Array<Condition> = []
//         for (let prop in obj) {
//             if (CustomProperties[prop])
//                 newCondtionArray.push(CustomProperties[prop](obj[prop]))
//             else if (isGenericCondition(obj[prop])) {
//                 newCondtionArray.push({
//                     field: prop,
//                     type: obj[prop].type,
//                     value: obj[prop].value
//                 })
//             } else {
//                 newCondtionArray.push({
//                     field: prop,
//                     type: ConditionType.equals,
//                     value: obj[prop]
//                 })
//             }
//         }
//         return newCondtionArray
//     }
//     function returnConditionsGroupFromGeneric(obj: any): ConditionGroup {
//         let newConditionGroup: ConditionGroup = {
//             type: obj.type,
//             conditions: []
//         }
//         evaluateConditions(obj.conditions, newConditionGroup)
//         return newConditionGroup
//     }
//     function evaluateConditions(genericConditions: Array<any>, conditionGroup: ConditionGroup): void {
//         genericConditions.forEach(condition => {
//             if (ConditionGroupClass.isConditionGroup(condition)) {
//                 conditionGroup.conditions.push(returnConditionsGroupFromGeneric(condition))
//             } else {
//                 conditionGroup.conditions.concat(returnConditionsArrayFromObject(condition))
//             }
//         })
//     }
//     let fields: Array<string> = CreatorParameters.customGroup.fields
//     let customWhere = CreatorParameters.customGroup.where
//     let newConditionGroup: ConditionGroup
//     let customConditions: Array<any>
//     if (!ConditionGroupClass.isConditionGroup(customWhere)) {
//         newConditionGroup = {
//             type: ConditionGroupType.and,
//             conditions: []
//         }
//         customConditions = customWhere.conditions
//     }
//     else {
//         newConditionGroup = {
//             type: customWhere.type,
//             conditions: []
//         }
//         customConditions = [customWhere]
//     }
//     evaluateConditions(customConditions, newConditionGroup)
//     return {
//         base: CreatorParameters.base,
//         fields,
//         where: newConditionGroup,
//         limit: CreatorParameters.limit
//     }
// }
//# sourceMappingURL=select.helper.class.js.map