import { query } from '../tools/access'
import fs from 'fs'

export class SelectTableCreator {
    constructor(table_name: string) {

        this.table_name = table_name
    }
    table_name: string

    table_properties: any

    imports: string = `
import { ConditionType, ConditionGroupType, NumberCondition, StringCondition, DateCondition, GenericSelectQuery, OrderByDirections, Condition } from './databasetool.interfaces'
import { SelectWizard } from './Table.class'
import { TableHelper } from './table.helper.class'
    `
    get fieldsEnum(): string {
        return this.createFieldsEnum()
    }

    get tableInterface(): string {
        return this.createTableInterface()
    }

    get staticRequirements(): string {
        return `

interface ${this.table_name}SelectQuery {
    fields: Array<${this.table_name}Fields>,
    where: ${this.table_name}ConditionGroup | ${this.table_name}Condition
}

interface ${this.table_name}ConditionGroup {
    type: ConditionGroupType,
    conditions: Array<${this.table_name}ConditionGroup | ${this.table_name}Condition> | ${this.table_name}Condition
}


interface ${this.table_name}SelectQuery {
    fields: Array<${this.table_name}Fields>,
    where: ${this.table_name}ConditionGroup | ${this.table_name}Condition,
    limit?: number,
    orderBy?: ${this.table_name}SelectOrderBy
}

interface ${this.table_name}SelectOrderBy {
    field: ${this.table_name}Fields,
    direction: OrderByDirections
}


let customFunctions: any = {
    someCustom: (input: any): Condition => {
        return {
            field: 'id',
            type: ConditionType.gt,
            value: input + ' AND THRN SOME'
        }
    }
}

export function ${this.table_name}Select(${this.table_name}Select: ${this.table_name}SelectQuery): SelectWizard {
    let genericSelect = ${this.table_name}Select as any as GenericSelectQuery
    let tableHelper = new TableHelper({ base: '${this.table_name}', customGroup: genericSelect, CustomProperties: customFunctions })
    return tableHelper.returnTable()
}

        `
    }

    async init() {
        this.table_properties = await query(`describe ${this.table_name}`)
    }

    private createFieldsEnum() {
        let create = (field: string): string => `${field} = '${field}',`
        let Fields: Array<String> = []
        this.table_properties.forEach((column: any) => {
            Fields.push(create(column.Field))
        });

        let insideEnums: String = Fields.join(`
        `)

        return `
    export enum ${this.table_name}Fields {
        ${insideEnums}
    }
        `

    }

    private createTableInterface() {
        let create = (field: string, type: string): string => {
            if (type.includes('int')) {
                return createNumberField(field)
            } else if (type.includes('text') || type.includes('varchar')) {
                return createStringField(field)
            } else if (type.includes('date') || type.includes('time')) {
                return createDateField(field)
            } else {
                console.log('\n\n\nERROR EVERY TYPE SHOULD BE ACCOUNTED FOR\n\n\n')
                return 'ERROR EVERY TYPE SHOULD BE ACCOUNTED FOR'
            }
        }
        let createNumberField = (field: string): string => `${field}?: number | NumberCondition,`
        let createDateField = (field: string): string => `${field}?: Date | DateCondition,`
        let createStringField = (field: string): string => `${field}?: String | StringCondition,`

        let Fields: Array<String> = []
        this.table_properties.forEach((column: any) => {
            Fields.push(create(column.Field, column.Type))
        });

        let insideEnums: String = Fields.join(`
        `)

        return `
    interface ${this.table_name}Condition {
        ${insideEnums}
    }
        `

    }

    get fullPage(): string {
        return `
    ${this.imports}
    ${this.fieldsEnum}
    ${this.tableInterface}
    ${this.staticRequirements}
        `
    }

    writeToFile(filePath: string): void {
        fs.writeFileSync(filePath, this.fullPage)
    }

}