import { ConditionType, ConditionGroupType, NumberCondition, StringCondition, DateCondition, GenericSelectQuery, OrderByDirections, Condition } from '../tools/databasetool.interfaces'
import { SelectWizard } from '../tools/select.class'
import { ConvertSpecificTableSelectIntoGeneralSelectWizard } from '../tools/select.helper.class'

export enum MessageFields {
    id = 'id',
    user = 'user',
    account = 'account',
    date_created = 'date_created',
    date_sent = 'date_sent',
    sent_to = 'sent_to',
    sent_from = 'sent_from',
    body = 'body',
    ourNumber = 'ourNumber',
    theirNumber = 'theirNumber',
}



interface MessageSelectQuery {
    fields: Array<MessageFields>,
    where: MessageConditionGroup | MessageCondition
}

interface MessageConditionGroup {
    type: ConditionGroupType,
    conditions: Array<MessageConditionGroup | MessageCondition> | MessageCondition
}

interface MessageCondition {
    id?: number | NumberCondition,
    user?: number | NumberCondition,
    account?: number | NumberCondition,
    date_created?: Date | DateCondition,
    date_sent?: Date | DateCondition,
    sent_to?: String | StringCondition,
    sent_from?: String | StringCondition,
    body?: String | StringCondition,
    ourNumber?: String | StringCondition,
    theirNumber?: String | StringCondition,
    someCustom?: any
}


interface MessageSelectQuery {
    fields: Array<MessageFields>,
    where: MessageConditionGroup | MessageCondition,
    limit?: number,
    orderBy?: MessageSelectOrderBy,
    groupBy?: string
}

interface MessageSelectOrderBy {
    field: MessageFields,
    direction: OrderByDirections
}


let customFunctions: any = {
    someCustom: (input: any): Condition => {
        return {
            field: 'id',
            type: ConditionType.gt,
            value: input + ' AND THRN SOME'
        }
    }
}

export function MessageSelect(messageSelect: MessageSelectQuery): SelectWizard {
    let genericSelect = messageSelect as any as GenericSelectQuery
    let tableHelper = new ConvertSpecificTableSelectIntoGeneralSelectWizard({ base: 'Messages', customGroup: genericSelect, CustomProperties: customFunctions })
    return tableHelper.returnTable()
}

