import { MessageSelect, MessageFields } from './messages.select.class'
import { SelectWizard } from '../tools/select.class'
import { OrderByDirections, StringConditionType } from '../tools/databasetool.interfaces';
function normalizeWS(s: any): String {
    s = s.trim()
    s = s.match(/\S+/g);
    return s ? s.join(' ') : '';
}
describe("Should run", () => {
    it("Should be awesome", () => {
        expect(1).toBe(1)
    })
})


describe("Should create new select Messages queries", () => {
    let { id, user, account, date_created, date_sent, body } = MessageFields
    let returnFromMessageSelect: SelectWizard = MessageSelect({
        fields: [id, user, account, date_created, body], where: {
            theirNumber: '+19794020005',
        },
        limit: 7,
        orderBy: {
            field: id,
            direction: OrderByDirections.asc
        }
    })
    it("Should be awesome", () => {
        expect(normalizeWS(returnFromMessageSelect.stringify())).toBe(normalizeWS("Select id , user , account , date_created , body From Messages Where ( theirNumber = '+19794020005' ) Order By id asc Limit 7"​​))
    })

    //Succeeds but no need to query the database constantly
    xit("should actually query database and return 7 results", async () => {
        let queryResults = await returnFromMessageSelect.execute()
        expect(queryResults).toHaveLength(7)
    })
})


describe("Should apply group by conditition", () => {
    let { id, user, account, date_created, date_sent, body } = MessageFields
    let returnFromMessageSelect: SelectWizard = MessageSelect({
        fields: [id, user, account, date_created, body], where: {
            body: {
                type: StringConditionType.like,
                value: 'Hello'
            },
        },
        limit: 100,
        orderBy: {
            field: id,
            direction: OrderByDirections.asc
        },
        groupBy: 'theirnumber'
    })
    it("Should be awesome", () => {
        expect(normalizeWS(returnFromMessageSelect.stringify())).toBe(normalizeWS("Select id , user , account , date_created , body From Messages Where ( body like '%Hello%' ) group by theirnumber Order By id asc Limit 100"​​))
    })

    //Succeeds but no need to query the database constantly
    xit("should actually query database and return 21 results", async () => {
        let queryResults = await returnFromMessageSelect.execute()
        expect(queryResults).toHaveLength(21)
    })


})

describe("Custom properties / functions should work", () => {
    let { id, user, account, date_created, date_sent, body } = MessageFields
    let returnFromMessageSelect: SelectWizard = MessageSelect({
        fields: [id, user, account, date_created, body], where: {
            theirNumber: '+19794020005',
            someCustom: 7

        },
        limit: 7,
        orderBy: {
            field: id,
            direction: OrderByDirections.asc
        }
    })
    it("Should be awesome", () => {
        expect(normalizeWS(returnFromMessageSelect.stringify())).toBe(normalizeWS("Select id , user , account , date_created , body From Messages Where ( theirNumber = '+19794020005' AND id > '7 AND THRN SOME' ) Order By id asc Limit 7"))
    })

})
