import { ConditionClass, ConditionGroupClass, SelectWizard } from '../tools/select.class'
import { ConditionType, ConditionGroupType, ConditionGroup, Condition } from '../tools/databasetool.interfaces';
function normalizeWS(s: any): String {
    s = s.match(/\S+/g);
    return s ? s.join(' ') : '';
}
let testCondition1: Condition = {
    type: ConditionType.gt,
    field: "Field 1",
    value: "Some Value"
}

let testCondition2: Condition = {
    type: ConditionType.equals,
    field: "Field 1",
    value: "Some Value"
}
let testCondition3: Condition = {
    type: ConditionType.lt,
    field: "Field 1",
    value: "Some Value"
}

let testCondition4: Condition = {
    type: ConditionType.like,
    field: "Field 1",
    value: "Some Value"
}

let testCondition5: Condition = {
    type: ConditionType.equals,
    field: "dateField",
    value: new Date('06-29-1984')
}

let testCondition6: Condition = {
    type: ConditionType.equals,
    field: "numberField",
    value: 32,
}



let simpleTable = new SelectWizard({
    base: 'Messages',
    fields: [
        'ourNumber',
        'theirnumber',
        'body'
    ],
    where: {
        field: 'theirnumber',
        type: ConditionType.equals,
        value: '+19794020005'
    },
    limit: 2
})


describe('ConditionClass and ConditionGroupClass', () => {




    it("ConditionClass.stringify() return the right string", done => {

        expect(new ConditionClass(testCondition1).stringify()).toBe(" Field 1  >  'Some Value'")

        expect(new ConditionClass(testCondition2).stringify()).toBe(" Field 1  =  'Some Value'")

        expect(new ConditionClass(testCondition3).stringify()).toBe(" Field 1  <  'Some Value'")

        expect(new ConditionClass(testCondition4).stringify()).toBe(` Field 1 like '%Some Value%'`)

        expect(new ConditionClass(testCondition5).stringify()).toBe(` dateField  =  '1984-06-29 00:00:00.000'`)

        expect(new ConditionClass(testCondition6).stringify()).toBe(` numberField  =  32`)

        done()
    })


    it("ConditionGroupClass.stringify() return the right string", done => {

        expect(new ConditionGroupClass({
            type: ConditionGroupType.and,
            conditions: [
                testCondition1,
                testCondition2,
                testCondition4
            ]
        }).stringify()).toBe(`
(
     Field 1  >  'Some Value'
        AND
         Field 1  =  'Some Value'
        AND
         Field 1 like '%Some Value%'
)`)


        expect(normalizeWS(new ConditionGroupClass({
            type: ConditionGroupType.and,
            conditions: [
                testCondition1,
                testCondition2,
                {
                    type: ConditionGroupType.or,
                    conditions: [
                        testCondition1,
                        testCondition2,
                        testCondition4
                    ]
                }]
        }).stringify())).toBe(normalizeWS(`
(
     Field 1  >  'Some Value'
        AND
         Field 1  =  'Some Value'
        AND

(
     Field 1  >  'Some Value'
        OR
         Field 1  =  'Some Value'
        OR
         Field 1 like '%Some Value%'
)
)`))

        done()
    })
})



describe('table class', () => {
    it("Should create the right query", () => {
        let newTable1 = new SelectWizard({
            base: 'Messages',
            fields: [
                'id',
                'account'
            ],
            where: {
                type: ConditionGroupType.and,
                conditions: [
                    testCondition1,
                    testCondition2,
                    {
                        type: ConditionGroupType.or,
                        conditions: [
                            testCondition1,
                            testCondition2,
                            testCondition4
                        ]
                    }]
            },
            groupBy: 'theirnumber'
        })
        expect(normalizeWS(newTable1.stringify())).toBe(normalizeWS(`
            Select
                id , account
            From
                Messages
            Where

    (
         Field 1  >  'Some Value'
            AND
             Field 1  =  'Some Value'
            AND

    (
         Field 1  >  'Some Value'
            OR
             Field 1  =  'Some Value'
            OR
             Field 1 like '%Some Value%'
    )
    )  group by theirnumber Limit 1000
            `))
    })



    xit('Should Query Database', async () => {

        expect(normalizeWS(simpleTable.stringify())).toBe(normalizeWS(`
            Select
                ourNumber , theirnumber , body
            From
                Messages
            Where

    (
         theirnumber  =  '+19794020005'
    )  Limit 2
            `))

        let queryResult: any = await simpleTable.execute()

        expect(queryResult.length).toBe(2)
    })
})
