import { GenericSelectQuery, SelectWizardProperties, ConditionGroup, Condition, ConditionType, ConditionGroupType, GenericOrderBy } from '../tools/databasetool.interfaces'

import { SelectWizard, ConditionGroupClass } from '../tools/select.class'

export class ConvertSpecificTableSelectIntoGeneralSelectWizard {

    constructor(CreatorParameters: { base: string, customGroup: GenericSelectQuery, CustomProperties?: any }) {

        this.base = CreatorParameters.base
        this.fields = CreatorParameters.customGroup.fields
        this.customFunctions = CreatorParameters.CustomProperties || {}
        this.where = this.generateWhere(CreatorParameters.customGroup.where)
        this.limit = CreatorParameters.customGroup.limit
        this.orderBy = CreatorParameters.customGroup.orderBy
        this.groupBy = CreatorParameters.customGroup.groupBy
    }
    groupBy: string
    fields: Array<String>
    customFunctions: any
    base: String
    where: ConditionGroup
    limit: number
    orderBy: GenericOrderBy
    get selectWizardProperties(): SelectWizardProperties {
        return {
            base: this.base,
            fields: this.fields,
            where: this.where,
            limit: this.limit,
            orderBy: this.orderBy,
            groupBy: this.groupBy
        }
    }


    generateWhere(inputWhere: any): ConditionGroup {
        return this.parseConditionOrConditionGroup(inputWhere)
    }

    parseConditionOrConditionGroup(conditionOrConditionGroup: any, parentConditionGroup?: ConditionGroup): ConditionGroup {
        let isConditionGroup: boolean = ConditionGroupClass.isConditionGroup(conditionOrConditionGroup)
        if (isConditionGroup && !parentConditionGroup) {
            return this.parseCondtionGroup(conditionOrConditionGroup)
        }
        else if (isConditionGroup && parentConditionGroup) {
            parentConditionGroup.conditions.push(this.parseCondtionGroup(conditionOrConditionGroup))
            return parentConditionGroup
        } else if (!isConditionGroup && !parentConditionGroup) {
            return this.returnNewCondtionGroup(conditionOrConditionGroup)
        } else if (!isConditionGroup && parentConditionGroup) {
            let ReturnCOnditos = this.parseCondtionToArrayOfConditions(conditionOrConditionGroup)
            parentConditionGroup.conditions = parentConditionGroup.conditions.concat(ReturnCOnditos)
            return parentConditionGroup
        } else {
            console.log('Should never be here, conditions should be met.')
        }
    }
    returnNewCondtionGroup(condition: any): ConditionGroup {
        return {
            type: ConditionGroupType.and,
            conditions: this.parseCondtionToArrayOfConditions(condition)
        }
    }
    parseCondtionGroup(conditionGroup: any): ConditionGroup {
        let newConditionGroup: ConditionGroup = {
            type: conditionGroup.type,
            conditions: []
        }
        conditionGroup.conditions.forEach((conditionOrConditionGroup: any) => {
            this.parseConditionOrConditionGroup(conditionOrConditionGroup, newConditionGroup)
        })
        return newConditionGroup
    }
    parseCondtionToArrayOfConditions(conditionGroup: any): Array<Condition> {
        let isGenericCondition = (n: any): boolean => n.type
        let newCondtionArray: Array<Condition> = []
        for (let prop in conditionGroup) {
            if (this.customFunctions[prop])
                newCondtionArray.push(this.customFunctions[prop](conditionGroup[prop]))
            else if (isGenericCondition(conditionGroup[prop])) {
                newCondtionArray.push({
                    field: prop,
                    type: conditionGroup[prop].type,
                    value: conditionGroup[prop].value
                })
            } else {
                newCondtionArray.push({
                    field: prop,
                    type: ConditionType.equals,
                    value: conditionGroup[prop]
                })
            }

        }
        return newCondtionArray
    }

    returnTable(): SelectWizard {
        return new SelectWizard(this.selectWizardProperties)
    }

}
//     static CustomCreator(CreatorParameters: { base: string, customGroup: GenericSelectQuery, CustomProperties?: any, limit?: number }): SelectWizardProperties {
//     let { CustomProperties } = CreatorParameters
//     if (!CustomProperties) CustomProperties = {}
//     function returnConditionsArrayFromObject(obj: any): Array<Condition> {
//         let isGenericCondition = (n: any): boolean => n.type
//         let newCondtionArray: Array<Condition> = []
//         for (let prop in obj) {
//             if (CustomProperties[prop])
//                 newCondtionArray.push(CustomProperties[prop](obj[prop]))
//             else if (isGenericCondition(obj[prop])) {
//                 newCondtionArray.push({
//                     field: prop,
//                     type: obj[prop].type,
//                     value: obj[prop].value
//                 })
//             } else {
//                 newCondtionArray.push({
//                     field: prop,
//                     type: ConditionType.equals,
//                     value: obj[prop]
//                 })
//             }

//         }
//         return newCondtionArray
//     }

//     function returnConditionsGroupFromGeneric(obj: any): ConditionGroup {
//         let newConditionGroup: ConditionGroup = {
//             type: obj.type,
//             conditions: []
//         }
//         evaluateConditions(obj.conditions, newConditionGroup)

//         return newConditionGroup
//     }

//     function evaluateConditions(genericConditions: Array<any>, conditionGroup: ConditionGroup): void {
//         genericConditions.forEach(condition => {
//             if (ConditionGroupClass.isConditionGroup(condition)) {
//                 conditionGroup.conditions.push(returnConditionsGroupFromGeneric(condition))
//             } else {
//                 conditionGroup.conditions.concat(returnConditionsArrayFromObject(condition))
//             }
//         })
//     }


//     let fields: Array<string> = CreatorParameters.customGroup.fields
//     let customWhere = CreatorParameters.customGroup.where

//     let newConditionGroup: ConditionGroup
//     let customConditions: Array<any>
//     if (!ConditionGroupClass.isConditionGroup(customWhere)) {
//         newConditionGroup = {
//             type: ConditionGroupType.and,
//             conditions: []
//         }
//         customConditions = customWhere.conditions
//     }
//     else {
//         newConditionGroup = {
//             type: customWhere.type,
//             conditions: []
//         }
//         customConditions = [customWhere]
//     }

//     evaluateConditions(customConditions, newConditionGroup)


//     return {
//         base: CreatorParameters.base,
//         fields,
//         where: newConditionGroup,
//         limit: CreatorParameters.limit
//     }
// }