import { ConvertSpecificTableSelectIntoGeneralSelectWizard } from '../tools/select.helper.class'
import { MessageFields } from './messages.select.class'
import { ConditionType } from '../tools/databasetool.interfaces';
let { id, user, account, date_created, body } = MessageFields

let testOne = {
    input: {
        base: 'Messages',
        customGroup: {
            fields: [id, user, account, date_created, body], where: {
                theirNumber: '+19794020005'
            }
        }
    },
    expectedOutput: {
        base: 'Messages',
        where: {
            type: 'AND',
            conditions: [{ "field": "theirNumber", "type": " = ", "value": "+19794020005" }]
        }
    }
}





describe("Should run table helper with testOne input object", () => {
    let newTableHelper: ConvertSpecificTableSelectIntoGeneralSelectWizard = new ConvertSpecificTableSelectIntoGeneralSelectWizard(testOne.input)
    it("fields property", () => {
        expect(newTableHelper.fields).toHaveLength(5)
        expect(newTableHelper.fields).toContain("id")
    })

    it("base", () => {
        expect(newTableHelper.base).toBe('Messages')
    })

    it("tablehelper where property have length of 1", () => {
        expect(newTableHelper.where.conditions[0]).toHaveProperty('type')
    })

    it("tablehelper where property type ==='AND'", () => {
        expect(newTableHelper.where.conditions).toHaveLength(testOne.expectedOutput.where.conditions.length)
    })
    it("tablehelper where property type ==='AND'", () => {
        expect(newTableHelper.where.type).toBe('AND')
    })

    it("Comoparing input out put objects where property", () => {
        expect(newTableHelper.where).toEqual(testOne.expectedOutput.where)
    })

})


let testTwo = {
    input: {
        base: 'Second_Table',
        customGroup: {
            fields: [id, user, account, date_created, body],
            where: {
                type: 'AND',
                conditions: [
                    { theirNumber: '+19794020005' },
                    {
                        theirNumber: '+19794020005',
                        ourNumber: 'OtherNumber'
                    },
                    {
                        body: {
                            type: ConditionType.like,
                            value: 'Lookng for something'
                        }
                    }
                ]
            }
        }
    },

    expectedOutput: {
        base: 'Second_Table',
        where: {
            "type": "AND",
            "conditions": [
                { "field": "theirNumber", "type": " = ", "value": "+19794020005" },
                { "field": "theirNumber", "type": " = ", "value": "+19794020005" },
                { "field": "ourNumber", "type": " = ", "value": "OtherNumber" },
                { "field": "body", "type": " like ", "value": "Lookng for something" }]
        }
    }
}

describe("Should run table helper with testTwo input object", () => {
    let newTableHelper: ConvertSpecificTableSelectIntoGeneralSelectWizard = new ConvertSpecificTableSelectIntoGeneralSelectWizard(testTwo.input)

    it("base", () => {
        expect(newTableHelper.base).toBe(testTwo.expectedOutput.base)
    })

    it("where", () => {
        expect(newTableHelper.where).toEqual(testTwo.expectedOutput.where)
    })

})



let testThree = {
    input: {
        base: 'Third_Table',
        customGroup: {
            fields: [id, user, account, date_created, body],
            where: {
                type: 'AND',
                conditions: [
                    { theirNumber: '+19794020005' },
                    {
                        theirNumber: '+19794020005',
                        ourNumber: 'OtherNumber'
                    },
                    {
                        body: {
                            type: ConditionType.like,
                            value: 'Lookng for something'
                        }
                    },
                    {
                        type: 'AND',
                        conditions: [
                            { theirNumber: '+' },
                            {
                                theirNumber: '+19794020005',
                                ourNumber: 'OtherNumber'
                            },
                            {
                                body: {
                                    type: ConditionType.like,
                                    value: 'Lookng for something'
                                }
                            }
                        ]
                    },
                    {
                        type: 'OR',
                        conditions: [
                            { theirNumber: '+ORNUMBER' },
                            {
                                theirNumber: '+ANother',
                                ourNumber: 'OtherNumber'
                            },
                            {
                                body: {
                                    type: ConditionType.like,
                                    value: 'OR ELSE'
                                }
                            }
                        ]
                    }
                ]
            }
        }
    },

    expectedOutput: {
        base: 'Third_Table',
        where: {
            "conditions":
                [
                    { "field": "theirNumber", "type": " = ", "value": "+19794020005" },
                    { "field": "theirNumber", "type": " = ", "value": "+19794020005" },
                    { "field": "ourNumber", "type": " = ", "value": "OtherNumber" },
                    { "field": "body", "type": " like ", "value": "Lookng for something" },
                    {
                        "conditions": [
                            { "field": "theirNumber", "type": " = ", "value": "+" },
                            { "field": "theirNumber", "type": " = ", "value": "+19794020005" },
                            { "field": "ourNumber", "type": " = ", "value": "OtherNumber" },
                            { "field": "body", "type": " like ", "value": "Lookng for something" }

                        ],
                        "type": "AND"
                    },
                    {
                        "conditions": [
                            { "field": "theirNumber", "type": " = ", "value": "+ORNUMBER" },
                            { "field": "theirNumber", "type": " = ", "value": "+ANother" },
                            { "field": "ourNumber", "type": " = ", "value": "OtherNumber" },
                            { "field": "body", "type": " like ", "value": "OR ELSE" }
                        ],
                        "type": "OR"
                    }],
            "type": "AND"
        }
    }

}


describe("Should run table helper with testTwo input object", () => {
    let newTableHelper: ConvertSpecificTableSelectIntoGeneralSelectWizard = new ConvertSpecificTableSelectIntoGeneralSelectWizard(testThree.input)

    it("base", () => {
        expect(newTableHelper.base).toBe(testThree.expectedOutput.base)
    })

    it("where", () => {
        expect(newTableHelper.where).toEqual(testThree.expectedOutput.where)
    })
})