import { ConditionType, ConditionGroupType, NumberCondition, StringCondition, DateCondition, GenericSelectQuery, OrderByDirections, Condition, GenericOrderBy } from '../tools/databasetool.interfaces'
import { ConvertSpecificTableSelectIntoGeneralSelectWizard } from '../tools/select.helper.class'
import { InstanceWizard } from '../tools/instance.class'


enum CouponsFields {
    star = '*',
    id = 'id',
    code = 'code',
    off = 'off',
    status = 'status',
    created_at = 'created_at',
    offType = 'offType',
}

let primary_key: CouponsFields = CouponsFields.id

interface CouponsCondition {
    id?: number | NumberCondition,
    code?: String | StringCondition,
    off?: number | NumberCondition,
    status?: number | NumberCondition,
    created_at?: Date | DateCondition,
    offType?: String | StringCondition,
}



interface CouponsConditionGroup {
    type: ConditionGroupType,
    conditions: Array<CouponsConditionGroup | CouponsCondition> | CouponsCondition
}


interface CouponsSelectOrderBy {
    field: CouponsFields,
    direction: OrderByDirections
}

interface CouponsSelectGroupBy {
    field: CouponsFields
}

let customFunctions: any = {
    someCustom: (input: any): Condition => {
        return {
            field: 'id',
            type: ConditionType.gt,
            value: input + ' AND THRN SOME'
        }
    }
}

interface CouponsInterface {
    id?: number,
    code?: string,
    off?: string,
    status?: string,
    offType?: string,
    created_at?: Date | string
}



export default class CouponsClass {

    fields = CouponsFields

    new(Coupons?: CouponsInterface): CouponsInstance {
        return new CouponsInstance(Coupons)
    }

    async select(CouponsSelect: {
        fields: Array<CouponsFields>,
        where: CouponsConditionGroup | CouponsCondition,
        limit?: number,
        orderBy?: CouponsSelectOrderBy,
        groupBy?: CouponsSelectGroupBy
    }): Promise<Array<any>> {
        let genericSelect = CouponsSelect as any as GenericSelectQuery
        if (!genericSelect.fields.includes(primary_key)) CouponsSelect.fields.push(primary_key)
        let tableHelper = new ConvertSpecificTableSelectIntoGeneralSelectWizard({ base: 'Coupons', customGroup: genericSelect, CustomProperties: customFunctions })
        let anyGenericResults = await tableHelper.returnTable().execute()
        let arrayOfGenericResults = anyGenericResults as any as Array<any>
        let arrayOfInstances: Array<CouponsInstance> = arrayOfGenericResults.map(l => new CouponsInstance(l))
        return arrayOfInstances
    }

    async upsert(Coupons: CouponsInterface | CouponsInstance | Array<CouponsInterface | CouponsInstance>): Promise<any> {
        let save = (single: any): Promise<any> => {
            if (single.isCouponsInstance)
                return single.save()
            else
                return new CouponsInstance(single).save()
        }
        if (Array.isArray(Coupons)) {
            return Promise.all(Coupons.map(el => save(el)))
        } else {
            return save(Coupons)
        }

    }


    async delete(Coupons: CouponsInterface | CouponsInstance | Array<CouponsInterface | CouponsInstance>): Promise<any> {
        let remove = (single: any): Promise<any> => {
            if (single.isCouponsInstance)
                return single.delete()
            else
                return new CouponsInstance(single).delete()
        }


        if (Array.isArray(Coupons)) {
            return Promise.all(Coupons.map(el => remove(el)))
        } else {
            return remove(Coupons)
        }
    }

    //Only shown if the primary key is a number
    async findById(id: number) {
        let newCondition: any = {}
        newCondition[primary_key] = id
        newCondition as CouponsCondition

        let resultOffind = await this.select({
            fields: [CouponsFields.star],
            where: newCondition
        })

        if (resultOffind.length && resultOffind.length === 1) return resultOffind[0]
        else return {
            error: "Should have returned an array with exactly 1 result, however it did not. please see the results property",
            results: resultOffind
        }
    }
}


class CouponsInstance {
    readonly isCouponsInstance: boolean = true
    constructor(Coupons?: CouponsInterface) {
        if (!Coupons) Coupons = {}
        let genericObj = Coupons as any
        this.instance = new InstanceWizard({ table: 'Coupons', primary_key: 'id', fields: genericObj })
    }
    private instance: InstanceWizard
    get id() { return this.instance.fields['id'] }
    set id(val: number) {
        if (this.instance.isNew) {
            this.instance.fields['id'] = val
        } else {
            console.log('YOU CAN NOT CHANGE THE PRIMARY KEY OF AN EXISTING ROW')
        }
    }
    get code(): string {
        return this.instance.fields['code']
    }
    get off(): string {
        return this.instance.fields['off']

    }
    get status(): string {
        return this.instance.fields['status']
    }
    get offType(): string {
        return this.instance.fields['offType']

    }
    get created_at(): Date | string {
        return this.instance.fields['created_at']
    }

    set code(val: string) {
        this.instance.fields['code'] = val
    }
    set off(val: string) {
        this.instance.fields['off'] = val

    }
    set status(val: string) {
        this.instance.fields['status'] = val
    }
    set offType(val: string) {
        this.instance.fields['offType'] = val

    }
    set created_at(val: Date | string) {
        this.instance.fields['created_at'] = val
    }

    async save(): Promise<any> {
        return await this.instance.save()
    }

    async delete(): Promise<any> {
        return await this.instance.delete()
    }
}