import Coupons from './coupon.example.class'
import { StringConditionType, NumberConditionType } from '../tools/databasetool.interfaces'

describe("It should have all the functionalities required", () => {
    let coupons = new Coupons()

    it("Should create the Coupons Class", () => {
        expect(coupons).toBeTruthy()
        expect(coupons).toHaveProperty('new')
        expect(coupons).toHaveProperty('select')
        expect(coupons).toHaveProperty('upsert')
        expect(coupons).toHaveProperty('delete')
    })

    //Success
    xit("Should be able to select", async () => {
        let selectResults = await coupons.select({
            fields: [
                coupons.fields.id,
                coupons.fields.offType,
                coupons.fields.code
            ], where: {
                id: {
                    type: NumberConditionType.gt,
                    value: 1
                }
            },
            limit: 2
        })
        expect(selectResults[0].code).toEqual("chesterlaw")
        expect(selectResults[0].id).toEqual(2)
        expect(selectResults[0].offType).toEqual("fixed")
        expect(selectResults[1].code).toEqual("Doesn't matter what is in this field")
        expect(selectResults[1].id).toEqual(9)

    })

    //success
    xit("Should be able to change and save one of the selected", async () => {
        let selectResults = await coupons.select({
            fields: [
                coupons.fields.id,
                coupons.fields.offType,
                coupons.fields.code
            ], where: {
                id: {
                    type: NumberConditionType.gt,
                    value: 1
                }
            },
            limit: 2
        })
        let randomCoupon = selectResults[1]
        let randomStringValue = "Rando" + Math.round(Math.random() * 1000)
        randomCoupon.offType = randomStringValue
        await randomCoupon.save()
        let returnNewInstanceOfRandomCoupon = await coupons.findById(randomCoupon.id)
        expect(returnNewInstanceOfRandomCoupon.offType).toEqual(randomStringValue)
    })

    //Success
    xit("Should always pull the primary key, regardless of user request", async () => {
        let selectResults: Array<any> = await coupons.select({
            fields: [
                coupons.fields.offType,
                coupons.fields.code
            ], where: {
                id: {
                    type: NumberConditionType.gt,
                    value: 1
                }
            },
            limit: 2
        })
        expect(selectResults[0]).toHaveProperty('id')
    })

    //success
    xit("Should be able to use the findById property", async () => {
        let coupon = await coupons.findById(9)
        expect(coupon).toEqual({ "code": "Doesn't matter what is in this field", "id": 9, "offType": null, "created_at": new Date("2018-08-17T18:52:49.000Z"), "status": 1, "off": null })
    })


    //Success
    xit("Should be able to insert a new coupon and save, when saving, the id should be added to the saved object", async () => {
        let coupon = coupons.new({ offType: "New OT", code: "Rando" + Math.round(Math.random() * 1000) })
        expect(coupon.id).toBeFalsy()
        await coupon.save()
        expect(coupon.id).toBeGreaterThan(1)
        console.log(coupon)
    })


    //Success
    xit("Should delete from the coupons class", async () => {
        let coupon = coupons.new({ offType: "New OT", code: "Rando" + Math.round(Math.random() * 1000) })
        expect(coupon.id).toBeFalsy()
        await coupon.save()
        expect(coupon.id).toBeGreaterThan(1)
        let res = await coupons.delete({ id: coupon.id })
        expect(res.affectedRows).toEqual(1)
    })


    //Success
    xit("Should delete from the instance", async () => {
        let coupon = coupons.new({ offType: "New OT", code: "Rando" + Math.round(Math.random() * 1000) })
        expect(coupon.id).toBeFalsy()
        await coupon.save()
        expect(coupon.id).toBeGreaterThan(1)
        let res = await coupon.delete()
        expect(res.affectedRows).toEqual(1)
    })



})

