export class TableCreator {
    constructor(private tableName: string, private tableProperties: any) {

        this.tableProperties.forEach((el: any) => {
            if (this.isPrimaryKey(el)) {
                this.primaryKey = el["Field"]
                this.primaryKeyType = this.typeOfRow(el)
            }
        });



    }

    primaryKey: string
    primaryKeyType: string

    isPrimaryKey(el: any): boolean {
        return el["Key"] === "PRI"
    }

    typeOfRow(el: any): string {
        return this.typescriptTypeFromMySqlType(el["Type"])
    }

    typescriptTypeFromMySqlType(mySqlType: string): string {
        if (mySqlType.includes('int') || mySqlType.includes('decimal')) {
            return 'number'
        } else if (mySqlType.includes('text') || mySqlType.includes('char')) {
            return 'string'
        } else if (mySqlType.includes('date') || mySqlType.includes('time')) {
            return 'Date'
        } else {
            console.log('\n\n\nERROR EVERY TYPE SHOULD BE ACCOUNTED FOR\n\n\n', 'TYPE:' + mySqlType + '\n\n')
            return 'ERROR EVERY TYPE SHOULD BE ACCOUNTED FOR'
        }
    }


    get imports(): string {
        let r =
            `import { ConditionType, ConditionGroupType, NumberCondition, StringCondition, DateCondition, GenericSelectQuery, OrderByDirections, Condition, GenericOrderBy } from '../tools/databasetool.interfaces'
import { ConvertSpecificTableSelectIntoGeneralSelectWizard } from '../tools/select.helper.class'
import { InstanceWizard } from '../tools/instance.class'

`
        return r
    }
    get fieldsEnum(): string {
        let create = (field: string): string => `${field} = '${field}',`
        let Fields: Array<String> = [`star = '*',`]
        this.tableProperties.forEach((column: any) => {
            Fields.push(create(column.Field))
        });

        let insideEnums: String = Fields.join(`
        `)

        return `
    export enum ${this.tableName}Fields {
        ${insideEnums}
    }
        `
    }

    get primaryKeyString(): string {
        if (this.primaryKey) {
            return `let primary_key: ${this.tableName}Fields = ${this.tableName}Fields['${this.primaryKey}']
`
        } else {
            return ''
        }

    }

    get condition(): string {
        let create = (field: string, type: string): string => {
            if (type === 'number') {
                return createNumberField(field)
            } else if (type === 'string') {
                return createStringField(field)
            } else if (type === 'Date') {
                return createDateField(field)
            } else {
                console.log('\n\n\nERROR EVERY TYPE SHOULD BE ACCOUNTED FOR\n\n\n')
                return 'ERROR EVERY TYPE SHOULD BE ACCOUNTED FOR'
            }
        }
        let createNumberField = (field: string): string => `${field}?: number | NumberCondition,`
        let createDateField = (field: string): string => `${field}?: Date | DateCondition,`
        let createStringField = (field: string): string => `${field}?: string | StringCondition,`

        let Fields: Array<String> = []
        this.tableProperties.forEach((column: any) => {
            let type = this.typescriptTypeFromMySqlType(column.Type)
            Fields.push(create(column.Field, type))
        });

        let insideEnums: String = Fields.join(`
        `)

        let r =
            `
interface ${this.tableName}Condition {
        ${insideEnums}
}
`
        return r
    }


    get simpleInterfacesAndCustomFunctions(): string {
        let r =
            `  
interface ${this.tableName}ConditionGroup {
    type: ConditionGroupType,
    conditions: Array<${this.tableName}ConditionGroup | ${this.tableName}Condition> | ${this.tableName}Condition
}


interface ${this.tableName}SelectOrderBy {
    field: ${this.tableName}Fields,
    direction: OrderByDirections
}

interface ${this.tableName}SelectGroupBy {
    field: ${this.tableName}Fields
}

let customFunctions: any = {
    someCustom: (input: any): Condition => {
        return {
            field: 'id',
            type: ConditionType.gt,
            value: input + ' AND THRN SOME'
        }
    }
}
`
        return r
    }

    get interface(): string {
        let create = (field: string, type: string): string => {
            if (type === 'number') {
                return createNumberField(field)
            } else if (type === 'string') {
                return createStringField(field)
            } else if (type === 'Date') {
                return createDateField(field)
            } else {
                console.log('\n\n\nERROR EVERY TYPE SHOULD BE ACCOUNTED FOR\n\n\n')
                return 'ERROR EVERY TYPE SHOULD BE ACCOUNTED FOR'
            }
        }
        let createNumberField = (field: string): string => `${field}?: number,`
        let createDateField = (field: string): string => `${field}?: Date,`
        let createStringField = (field: string): string => `${field}?: String,`

        let Fields: Array<String> = []
        this.tableProperties.forEach((column: any) => {
            let type = this.typescriptTypeFromMySqlType(column.Type)
            Fields.push(create(column.Field, type))
        });

        let insideEnums: String = Fields.join(`
        `)

        let r =
            `
interface ${this.tableName}Interface {
        ${insideEnums}
}
        `
        return r
    }

    get tableClass(): string {
        let r =
            `
export default class ${this.tableName}Class {

fields = ${this.tableName}Fields

new(${this.tableName}?: ${this.tableName}Interface): ${this.tableName}Instance {
    return new ${this.tableName}Instance(${this.tableName})
}

async select(${this.tableName}Select: {
    fields: Array<${this.tableName}Fields>,
    where: ${this.tableName}ConditionGroup | ${this.tableName}Condition,
    limit?: number,
    orderBy?: ${this.tableName}SelectOrderBy,
    groupBy?: ${this.tableName}SelectGroupBy
}): Promise<Array<any>> {
    let genericSelect = ${this.tableName}Select as any as GenericSelectQuery
    if (!genericSelect.fields.includes(primary_key)) ${this.tableName}Select.fields.push(primary_key)
    let tableHelper = new ConvertSpecificTableSelectIntoGeneralSelectWizard({ base: '${this.tableName}', customGroup: genericSelect, CustomProperties: customFunctions })
    let anyGenericResults = await tableHelper.returnTable().execute()
    let arrayOfGenericResults = anyGenericResults as any as Array<any>
    let arrayOfInstances: Array<${this.tableName}Instance> = arrayOfGenericResults.map(l => new ${this.tableName}Instance(l))
    return arrayOfInstances
}

async upsert(${this.tableName}: ${this.tableName}Interface | ${this.tableName}Instance | Array<${this.tableName}Interface | ${this.tableName}Instance>): Promise<any> {
    let save = (single: any): Promise<any> => {
        if (single.is${this.tableName}Instance)
            return single.save()
        else
            return new ${this.tableName}Instance(single).save()
    }
    if (Array.isArray(${this.tableName})) {
        return Promise.all(${this.tableName}.map(el => save(el)))
    } else {
        return save(${this.tableName})
    }

}


async delete(${this.tableName}: ${this.tableName}Interface | ${this.tableName}Instance | Array<${this.tableName}Interface | ${this.tableName}Instance>): Promise<any> {
    let remove = (single: any): Promise<any> => {
        if (single.is${this.tableName}Instance)
            return single.delete()
        else
            return new ${this.tableName}Instance(single).delete()
    }


    if (Array.isArray(${this.tableName})) {
        return Promise.all(${this.tableName}.map(el => remove(el)))
    } else {
        return remove(${this.tableName})
    }
}

${this.findByIdString}

}
`
        return r
    }


    get findByIdString(): string {
        if (this.primaryKey && this.primaryKeyType === 'number') {
            return `
        //Only shown if the primary key is a number
async findById(id: number) {
    let newCondition: any = {}
    newCondition[primary_key] = id
    newCondition as ${this.tableName}Condition

    let resultOffind = await this.select({
        fields: [${this.tableName}Fields.star],
        where: newCondition
    })

    if (resultOffind.length && resultOffind.length === 1) return resultOffind[0]
    else return {
        error: "Should have returned an array with exactly 1 result, however it did not. please see the results property",
        results: resultOffind
    }
}
            `
        } else {
            return ''
        }
    }

    get instanceClass(): string {
        return `
class ${this.tableName}Instance {
    readonly is${this.tableName}Instance: boolean = true
    constructor(${this.tableName}?: ${this.tableName}Interface) {
        if (!${this.tableName}) ${this.tableName} = {}
        let genericObj = ${this.tableName} as any
        this.instance = new InstanceWizard({ table: '${this.tableName}', primary_key: 'id', fields: genericObj })
    }
    private instance: InstanceWizard

    ${this.createGettersAndSettersForInstance}

    async save(): Promise<any> {
        return await this.instance.save()
    }

    async delete(): Promise<any> {
        return await this.instance.delete()
    }
}
        
        `
    }


    get createGettersAndSettersForInstance(): string {
        let gettersAndSetters: Array<string> = []

        this.tableProperties.forEach((el: any) => {
            let field = el["Field"]
            let type = this.typeOfRow(el)
            if (type === 'Date') type = 'Date | string'

            if (this.isPrimaryKey(el)) {
                gettersAndSetters.push(`
get ${field}() { return this.instance.fields['${field}'] }
set ${field}(val: ${type}) {
    if (this.instance.isNew) {
        this.instance.fields['${field}'] = val
    } else {
        console.log('YOU CAN NOT CHANGE THE PRIMARY KEY OF AN EXISTING ROW')
    }
}
                `)
            } else {
                gettersAndSetters.push(`
get ${field}(): ${type} {
    return this.instance.fields['${field}']
}

    set ${field}(val: ${type}) {
    this.instance.fields['${field}'] = val
}
                `)


            }
        })

        return gettersAndSetters.join(`
        `)
    }



    get fullPage(): string {
        return `
    ${this.imports}
    ${this.fieldsEnum}
    ${this.primaryKeyString}
    ${this.condition}
    ${this.simpleInterfacesAndCustomFunctions}
    ${this.interface}
    ${this.tableClass}
    ${this.instanceClass}
        `
    }

}