import MasterCreator from './creator/master.creator.class'
import { TableCreator } from './creator/table.creator.class'
import { } from './tools/databasetool.interfaces'
import fs from 'fs'

export class mysqlTypescript {

    constructor(private db: any, private _directory: string, private databaseSchemaName: string) {
        this.createFoldersIfnotExists()
        this.copyToolsFolder()
    }

    private _tableNames: Array<string>
    async getTableNames() {
        if (this._tableNames) return this._tableNames
        let tableNames: any = await this.query(`
        SELECT table_name 
        FROM information_schema.tables 
        WHERE table_schema = '${this.databaseSchemaName}'
        `)
        this._tableNames = tableNames.map((tableName: any) => tableName.table_name)
        return this._tableNames
    }

    createFoldersIfnotExists(): void {
        this.checkAndMakeDir(`${this._directory}/database`)
        this.checkAndMakeDir(`${this._directory}/database/tables`)
        this.checkAndMakeDir(`${this._directory}/database/tools`)
        this.checkAndMakeDir(`${this._directory}/database/functions`)
    }

    copyToolsFolder(): void {
        let arrayOfTools: Array<any> = fs.readdirSync(`${__dirname}/tools`).map(file => ({
            source: `${__dirname}/tools/${file}`,
            destination: `${this._directory}/database/tools/${file}`
        }))

        for (let tool of arrayOfTools) {
            fs.copyFileSync(tool.source, tool.destination)
        }
    }



    async query(q: string, parameters?: any) {
        return new Promise((resolve, reject) => {
            this.db.query(q, parameters, (error: any, response: any) => {
                if (!error) {
                    resolve(response)
                } else {
                    reject(error);
                }
            })
        })
    }



    async checkAndMakeDir(dir: string) {
        if (fs.existsSync(dir))
            return
        else
            fs.mkdirSync(dir)
    }
    async CreateTable(tableName: string) {
        let tableProperties = await this.query(`Describe ${tableName}`)
        let fullPageString = new TableCreator(tableName, tableProperties).fullPage
        let pathToWrite: string = `${this._directory}/database/tables/${tableName}.class.ts`
        return fs.writeFileSync(pathToWrite, fullPageString)
    }

    async CreateAllTables() {
        let tableNames: any = await this.getTableNames()
        for (let table of tableNames) {
            await this.CreateTable(table)
        }
    }
    async CreateFunctions() {
        return
    }

    async CreateMaster() {
        let tables: Array<string> = await this.getTableNames()
        let nmc = new MasterCreator(tables)
        let pathToWrite: string = `${this._directory}/database/master.class.ts`
        fs.writeFileSync(pathToWrite, nmc.fullPage)
    }

    async CreateAll() {
        await this.CreateAllTables()
        await this.CreateFunctions()
        return await this.CreateMaster()
    }

}

