export interface SelectWizardProperties {
    base: String,
    fields: Array<String>,
    where: ConditionGroup | Condition,
    limit?: number,
    orderBy?: GenericOrderBy,
    groupBy?: string
}


export interface SelectQuery {
    fields: Array<String>,
    where: ConditionGroup | Condition
}


export enum ConditionGroupType {
    and = "AND",
    or = "OR"
}

export interface Condition {
    field: String,
    type: ConditionType,
    value: String | number | Date | Array<String | number | Date>
}

export enum ConditionType {
    equals = ' = ',
    gt = ' > ',
    lt = ' < ',
    gtoe = ' >= ',
    ltoe = ' <= ',
    like = ' like ',
    between = ' between '
}

export interface ConditionGroup {
    type: ConditionGroupType,
    conditions: Array<ConditionGroup | Condition>
}

export interface SelectTableMakerFunction {
    (base: string, select: Function): SelectWizardProperties
}

export enum NumberConditionType {
    equals = ConditionType.equals,
    gt = ConditionType.gt,
    lt = ConditionType.lt,
    gtoe = ConditionType.gtoe,
    ltoe = ConditionType.ltoe
}

export enum StringConditionType {
    equals = ConditionType.equals,
    like = ConditionType.like
}

export enum DateConditionType {
    equals = ConditionType.equals,
    gt = ConditionType.gt,
    lt = ConditionType.lt,
    gtoe = ConditionType.gtoe,
    ltoe = ConditionType.ltoe,
    between = ConditionType.between
}

export interface NumberCondition {
    type: NumberConditionType,
    value: String | number | Date
}


export interface StringCondition {
    type: StringConditionType,
    value: String,
}


export interface DateCondition {
    type: StringConditionType,
    value: String | Date | Array<String | number | Date>
}

export interface GenericSelectQuery {
    fields: Array<string>,
    where: any,
    limit?: number,
    orderBy?: GenericOrderBy,
    groupBy?: string
}

export interface GenericOrderBy {
    field: String,
    direction: OrderByDirections
}

export enum OrderByDirections {
    asc = ' asc ',
    desc = ' desc '
}