import { SelectWizardProperties, ConditionGroup, Condition, ConditionGroupType, ConditionType, GenericOrderBy } from './databasetool.interfaces'
import { query, db } from './access'
export class SelectWizard {
    constructor(selectWizardProperties: SelectWizardProperties) {
        if (ConditionGroupClass.isConditionGroup(selectWizardProperties.where))
            this.conditions = new ConditionGroupClass(selectWizardProperties.where)
        else
            this.conditions = new ConditionGroupClass({
                type: ConditionGroupType.and,
                conditions: [selectWizardProperties.where]
            })
        this.fields = selectWizardProperties.fields
        this.base = selectWizardProperties.base
        this.orderBy = selectWizardProperties.orderBy
        this.groupBy = selectWizardProperties.groupBy
        if (selectWizardProperties.limit) this.limit = selectWizardProperties.limit
    }
    limit: number = 1000
    base: String
    fields: Array<String>
    conditions: ConditionGroupClass
    orderBy: GenericOrderBy
    groupBy: string
    fieldsString(): String { return this.fields.join(' , ') }
    getOrderByString() {
        if (this.orderBy) {
            let { field, direction } = this.orderBy
            return ` Order By ${field} ${direction} `
        } else return ''
    }


    getGroupByString() {
        if (this.groupBy) {
            return ` group by ${this.groupBy} `
        } else {
            return ''
        }
    }

    public async  execute() {
        return await query(this.stringify())
    }
    public stringify(): string {

        return `
        Select
            ${this.fieldsString()}
        From
            ${this.base}
        Where
            ${this.conditions.stringify()}
        ${this.getGroupByString()}
        ${this.getOrderByString()}
        Limit
            ${this.limit}
        `
    }
}



export class ConditionGroupClass {
    constructor(conditionGroup: ConditionGroup) {
        this.type = conditionGroup.type
        conditionGroup.conditions.forEach(condition => {
            if (ConditionGroupClass.isConditionGroup(condition))
                this.conditions.push(new ConditionGroupClass(condition))
            else
                this.conditions.push(new ConditionClass(condition))
        })
    }
    private type: ConditionGroupType
    private conditions: Array<ConditionGroupClass | ConditionClass> = []

    static isConditionGroup(arg: any): arg is ConditionGroup {
        return arg.conditions ? true : false
    }
    private returnArrayOfConditionStrings(): Array<String> {
        return this.conditions.map(condition => condition.stringify())
    }
    public stringify(): String {
        let arrayOfConditionStrings: Array<String> = this.returnArrayOfConditionStrings()
        let joinedConditions: String = arrayOfConditionStrings.join(`
        ${this.type}
        `)
        return `
(
    ${joinedConditions}
)`
    }
}

export class ConditionClass {
    constructor(private condition: Condition) { }
    public stringify(): String {
        switch (this.condition.type) {
            case ConditionType.equals:
            case ConditionType.gt:
            case ConditionType.lt:
            case ConditionType.ltoe:
            case ConditionType.gtoe:
                return ` ${this.condition.field} ${this.condition.type} ${db.escape(this.condition.value)}`
                break;
            case ConditionType.like:
                return ` ${this.condition.field} like ${db.escape(`%${this.condition.value}%`)}`
                break

            default:
                return 'SOMETHING WENT WRONG'
                break;
        }
    }
}




